function [gscore,oscore,rscore,nscore] = subsampling_scores(X,A,distances_geodesic,omega,K)

% Obtain scores of a given subsampling for greedy and optimization algorithm
% Also return the residual variances and neighborliness performance metrics

% Inputs: Data X, subsampling omega, number of neighbors K
% Outputs: scores (Greedy, Optimization, Residual-variance, Neighborliness)

% Subsample dataset and secants
Xs = X(omega,:);
As = A(omega,:);

% Compute greedy score: average difference of squared norms of secants
% before and after subsampling

gscore = mean((sum(As,1)-(length(omega)/size(A,1))).^2);

% Compute optimization score: maximum difference of squared norm of secants
% before and after subsampling

oscore = max(abs(sum(As,1)-(length(omega)/size(A,1))));

% Perform manifold learning on subsampled manifold

distances_subsampled = L2_distance(Xs, Xs, 1);
[Y_isomap,R_isomap,E_isomap] = Isomap2(distances_subsampled,'k',K);

% Compute residual variance
dim = length(Y_isomap.coords);
rscore = zeros(1,dim);
for di=1:dim,
    distances_subsampled_manifold = L2_distance(Y_isomap.coords{di},Y_isomap.coords{di},1);
    r2 = 1-corrcoef(distances_geodesic(:),distances_subsampled_manifold(:)).^2;
    rscore(di) = r2(2,1);
end

%max_neighbors = 10;
max_neighbors = 5;

% Compute neighborliness metric
nscore = zeros(1,max_neighbors);
distances_subsampled_manifold = L2_distance(Y_isomap.coords{3},Y_isomap.coords{3},1);
for i=1:max_neighbors,
    for j=1:size(distances_geodesic,1),
        neighbors_full_geodesic = GeodesicNearestNeighbors(distances_geodesic,j, 10*i);
        neighbors_subsampled_manifold = GeodesicNearestNeighbors(distances_subsampled_manifold,j, 10*i);
        nscore(i) = nscore(i) + 10*i - length(setdiff(neighbors_full_geodesic,neighbors_subsampled_manifold));
    end
end
nscore = nscore./(10*size(X,2)*(1:max_neighbors));