
%   greedy_algo: finds the masking pattern according to the MAPS-Isomap 
%   algorithm as described in [MAPS].
%
%   Inputs
%   A : normalized secants matrix
%   m : masking size
%
%   Output
%   omega_greedy: masking index set found according to MAPS-Isomap
%   algorithm
%
%   BEGIN COPYRIGHT NOTICE
%
%   MAPS code -- (c) 2013-2014 Hamid Dadkhahi
%
%    This code is provided as is, with no guarantees except that 
%    bugs are almost surely present.  Published reports of research 
%    using this code (or a modified version) should cite the 
%    article that describes the algorithm: 
%
%      [MAPS] H. Dadkhahi, M. F. Duarte, Masking Strategies for Image
%      Manifolds, 2014.
%
%    Comments and bug reports are welcome.  Email to hdadkhahi@ecs.umass.edu. 
%    I would also appreciate hearing about how you used this code, 
%    improvements that you have made to it, or translations into other
%    languages.    
%
%    You are free to modify, extend or distribute this code, as long 
%    as this copyright notice is included whole and unchanged. 
%
%    END COPYRIGHT NOTICE

function omega_greedy = greedy_algo(A,m)

C = A';
d = size(A,1);
omega = 1:d;
omega_greedy = [];

for i = 1 : m
    temp = repmat(sum(C(:,omega_greedy),2),1,length(omega));
    [~,index] = min(sum(abs(C(:,omega) + temp - (i/d)),1));
    omega_greedy = [omega_greedy omega(index)]; %#ok<AGROW>
    omega(index) = [];
end