
%   manifold_masking_Isomap_main.m: reproduces the plots given in Figures 2
%   and 3 of [MAPS].
%
%   BEGIN COPYRIGHT NOTICE
%
%   MAPS code -- (c) 2013-2014 Hamid Dadkhahi
%
%    This code is provided as is, with no guarantees except that 
%    bugs are almost surely present.  Published reports of research 
%    using this code (or a modified version) should cite the 
%    article that describes the algorithm: 
%
%      [MAPS] H. Dadkhahi, M. F. Duarte, Masking Strategies for Image
%      Manifolds, 2014.
%
%    Comments and bug reports are welcome.  Email to hdadkhahi@ecs.umass.edu. 
%    I would also appreciate hearing about how you used this code, 
%    improvements that you have made to it, or translations into other
%    languages.    
%
%    You are free to modify, extend or distribute this code, as long 
%    as this copyright notice is included whole and unchanged. 
%
%    END COPYRIGHT NOTICE

clear all;

warning off; %#ok<WNOFF>

% choose the dataset
dataset = 'Eyeglasses';
%dataset = 'MNIST';
%dataset = 'Heads';  
%dataset = 'Faces';  
%dataset = 'Statue';

% set the parameters according to the chosen dataset
if strcmp(dataset,'Eyeglasses')
    load X_eyedata.mat;
    K_LLE = 12;
    K_Isomap = 12;
    n_dimensions = 2;
elseif strcmp(dataset,'Heads')
    load X_faces.mat;
    K_LLE = 7;
    K_Isomap = 10;
    n_dimensions = 3;
elseif strcmp(dataset,'Faces')
    load X_faces_LLE.mat;
    K_LLE = 10;
    K_Isomap = 9;
    n_dimensions = 3;
elseif strcmp(dataset,'MNIST')
    load X_MNIST.mat;
    K_LLE = 12;
    K_Isomap = 10;
    n_dimensions = 5;
elseif strcmp(dataset,'Statue')
    load X_Statue.mat;
    K_LLE = 11;
    K_Isomap = 8;
    n_dimensions = 3;
else
    display('dataset not found!');
end

X = double(X);
%X = X / max(X(:));

d = size(X,1);
n = size(X,2);

nn_index = 2;

m = 50:50:300;
m_low = min(m);
m_high = max(m);

% PCoA 
omega_PCoA = generate_omega_PCoA(X,m_high);

% Isomap
distances_ambient = L2_distance(X, X, 1);
[~,~,E_isomap,distances_geodesic] = Isomap2(distances_ambient,'k',K_Isomap);

% Generating the secants matrix from neighborhood
[x_ind,y_ind] = find( triu(double(E_isomap) - eye(size(E_isomap,1))) );
A = ( X(:,x_ind) - X(:,y_ind) ).^2;
normA = sum(A,1);
A = A./repmat(normA,d,1);

% Greedy Isomap
omega_Isomap = greedy_algo(A,m(end));

% Isomap scores
[~,~,rv_original,nn_original] = subsampling_scores(X,A,distances_geodesic,1:d,K_Isomap);
rv_full = repmat(rv_original(n_dimensions),1,length(m));
nn_full = repmat(nn_original(nn_index),1,length(m));

rv_random = zeros(1,length(m));
rv_Isomap = zeros(1,length(m));
rv_LLE = zeros(1,length(m));
rv_PCA = zeros(1,length(m));
rv_PCoA = zeros(1,length(m));
rv_SPCA = zeros(1,length(m));

nn_random = zeros(1,length(m));
nn_Isomap = zeros(1,length(m));
nn_LLE = zeros(1,length(m));
nn_PCA = zeros(1,length(m));
nn_PCoA = zeros(1,length(m));
nn_SPCA = zeros(1,length(m));

% LLE algo
omega_LLE = LLE_graph_algo(X,K_LLE,m(end));

for i = 1 : length(m)
    perccount(i,length(m))
    
    % Random
    T = 50;

    rscore_temp = zeros(T,10);
    %nscore_temp = zeros(T,10);
    nscore_temp = zeros(T,5);

    for t = 1:T
        omega_random = randperm(d,m(i));
        try
            [~,~,rscore_temp(t,:),nscore_temp(t,:)] = subsampling_scores(X,A,distances_geodesic,omega_random,K_Isomap);
        catch %#ok<CTCH>
            t = t - 1; %#ok<FXSET>
            continue
        end
    end

    rscore_random = sum(rscore_temp)/T;    
    rv_random(i) = rscore_random(n_dimensions);
    
    nscore_random = sum(nscore_temp)/T;      
    nn_random(i) = nscore_random(nn_index);
    
    % Isomap masking scores
    [~,~,rscore_isomap,nscore_isomap] = subsampling_scores(X,A,distances_geodesic,omega_Isomap(1:m(i)),K_Isomap);
    rv_Isomap(i) = rscore_isomap(n_dimensions);
    nn_Isomap(i) = nscore_isomap(nn_index);
    
    % LLE scores
    [~,~,rscore_LLE,nscore_LLE] = subsampling_scores(X,A,distances_geodesic,omega_LLE(1:m(i)),K_Isomap);
    rv_LLE(i) = rscore_LLE(n_dimensions);
    nn_LLE(i) = nscore_LLE(nn_index);

    % PCoA scores
    [~,~,rscore_PCoA,nscore_PCoA] = subsampling_scores(X,A,distances_geodesic,omega_PCoA(1:m(i)),K_Isomap);
    rv_PCoA(i) = rscore_PCoA(n_dimensions);
    nn_PCoA(i) = nscore_PCoA(nn_index);
    
    % PCA scores
    [U,S,V] = svds(X,m(i));
    X_PCA = U'*X;
    [rscore_PCA,nscore_PCA] = embedding_scores(X_PCA,distances_geodesic);
    rv_PCA(i) = rscore_PCA(n_dimensions);
    nn_PCA(i) = nscore_PCA(nn_index);
    
    % SparsePCA scores
    [cards,vars,Z]= sparsePCA(X',m(i));
    omega_SPCA = find( abs(Z) > 0.01 );
    [~,~,rscore_SPCA,nscore_SPCA] = subsampling_scores(X,A,distances_geodesic,omega_SPCA(1:m(i)),K_Isomap);
    rv_SPCA(i) = rscore_SPCA(n_dimensions);
    nn_SPCA(i) = nscore_SPCA(nn_index);
    
end

% RV plots
figure, plot(m,rv_full,'Color','b','LineStyle','--','Marker','x','LineWidth',2)
hold on, plot(m,rv_PCA,'Color','c','LineStyle','-.','Marker','p','LineWidth',2)
hold on, plot(m,rv_SPCA,'Color',[1 0.5 0],'LineStyle','-','Marker','d','LineWidth',2)
hold on, plot(m,rv_PCoA,'Color','g','LineStyle','-','Marker','none','LineWidth',2)
hold on, plot(m,rv_LLE,'Color','r','LineStyle','-','Marker','+','LineWidth',2)
hold on, plot(m,rv_Isomap,'Color','k','LineStyle','-','Marker','v','LineWidth',2)
hold on, plot(m,rv_random,'Color','m','LineStyle','-','Marker','o','LineWidth',2)
axis tight
axisfortex('','Embedding Dim./Masking Size m','Residual Variance')
%legend('Full Data','PCA','SPCA','PCoA','MAPS-LLE','MAPS-Isomap','Random','Location','NorthEast');
title(dataset)

% NN Plots
figure, plot(m,100*nn_full,'Color','b','LineStyle','--','Marker','x','LineWidth',2)
hold on, plot(m,100*nn_PCA,'Color','c','LineStyle','-.','Marker','p','LineWidth',2)
hold on, plot(m,100*nn_SPCA,'Color',[1 0.5 0],'LineStyle','-','Marker','d','LineWidth',2)
hold on, plot(m,100*nn_PCoA,'Color','g','LineStyle','-','Marker','none','LineWidth',2)
hold on, plot(m,100*nn_LLE,'Color','r','LineStyle','-','Marker','+','LineWidth',2)
hold on, plot(m,100*nn_Isomap,'Color','k','LineStyle','-','Marker','v','LineWidth',2)
hold on, plot(m,100*nn_random,'Color','m','LineStyle','-','Marker','o','LineWidth',2)
axis tight
axisfortex('','Embedding Dim./Masking Size m','% Preserved Neighbors')
%legend('Full Data','PCA','SPCA','PCoA','MAPS-LLE','MAPS-Isomap','Random','Location','SouthEast');
title(dataset)
