
%   LLE_experiments.m: reproduces the plots given in Figure 4 of [MAPS].
%
%   BEGIN COPYRIGHT NOTICE
%
%   MAPS code -- (c) 2013-2014 Hamid Dadkhahi
%
%    This code is provided as is, with no guarantees except that 
%    bugs are almost surely present.  Published reports of research 
%    using this code (or a modified version) should cite the 
%    article that describes the algorithm: 
%
%      [MAPS] H. Dadkhahi, M. F. Duarte, Masking Strategies for Image
%      Manifolds, 2014.
%
%    Comments and bug reports are welcome.  Email to hdadkhahi@ecs.umass.edu. 
%    I would also appreciate hearing about how you used this code, 
%    improvements that you have made to it, or translations into other
%    languages.    
%
%    You are free to modify, extend or distribute this code, as long 
%    as this copyright notice is included whole and unchanged. 
%
%    END COPYRIGHT NOTICE

clear all;

warning off; %#ok<WNOFF>

% choose the dataset 
dataset = 'Eyeglasses';
%dataset = 'MNIST';
%dataset = 'Heads';        
%dataset = 'Faces';   
%dataset = 'Squares';
%dataset = 'Circles';
%dataset = 'Statue';

% set the parameters according to the chosen dataset
if strcmp(dataset,'Eyeglasses')
    load X_eyedata.mat;
    K_LLE = 12;
    K_Isomap = 12;
    n_dimensions = 2;
elseif strcmp(dataset,'Heads')
    load X_faces.mat;
    K_LLE = 7;    
    K_Isomap = 10;
    n_dimensions = 3;
elseif strcmp(dataset,'Faces')
    load X_faces_LLE.mat;
    K_LLE = 10;
    K_Isomap = 9;
    n_dimensions = 3;
elseif strcmp(dataset,'MNIST')
    load X_MNIST.mat;
    K_LLE = 12;
    K_Isomap = 10;
    n_dimensions = 5;
elseif strcmp(dataset,'Squares')
    load X_Squares;
    K_LLE = 10;
    K_Isomap = 10;
    n_dimensions = 2;  
elseif strcmp(dataset,'Circles')
    load X_circles.mat
    K_LLE = 10;
    K_Isomap = 10;
    n_dimensions = 2;
elseif strcmp(dataset,'Statue')
    load X_Statue.mat;
    K_LLE = 11;
    K_Isomap = 8;
    n_dimensions = 3;
else
    display('dataset not found!');
end

X = double(X);
%X = X / max(X(:));

d = size(X,1);
n = size(X,2);

m = 50:50:300;
%m = 100;

m_high = max(m);
m_low = min(m);

% PCoA
omega_PCoA = generate_omega_PCoA(X,m_high);

% LLE Masking Algorithm
omega_LLE = LLE_graph_algo(X,K_LLE,m_high);

% Isomap Masking Algorithm
distances_ambient = L2_distance(X, X, 1);
[Y_isomap,R_isomap,E_isomap,distances_geodesic] = Isomap2(distances_ambient,'k',K_Isomap);
[x_ind,y_ind] = find( triu(double(E_isomap) - eye(size(E_isomap,1))) );
A = ( X(:,x_ind) - X(:,y_ind) ).^2;
normA = sum(A,1);
A = A./repmat(normA,d,1);
omega_Isomap = greedy_algo(A,m_high);
clear A normA x_ind y_ind E_isomap;

% Computing the error

[Y_full,W,neighborhood] = lle_with_weights(X,K_LLE,n_dimensions);

error_full = 0;
error_PCA = zeros(size(m));
error_LLE = zeros(size(m));
error_Isomap = zeros(size(m));
error_random = zeros(size(m));
error_PCoA = zeros(size(m));
error_SPCA = zeros(size(m));

% numebr of iterations for random masking
T = 50;

for j = 1 : n
    error_full = error_full + norm(Y_full(:,j) ...
        - sum(repmat(W(:,j)',n_dimensions,1).*Y_full(:,neighborhood(:,j)),2),2)^2;
end

for i = 1:length(m) 
    perccount(i,length(m))
    
    [U,S,V] = svds(X,m(i));
    X_PCA = U'*X;
    [Y_PCA,~,~] = lle_with_weights(X_PCA,K_LLE,n_dimensions);
    for j = 1 : n
        error_PCA(i) = error_PCA(i) + norm(Y_PCA(:,j) - ...
            sum(repmat(W(:,j)',n_dimensions,1).*Y_PCA(:,neighborhood(:,j)),2),2)^2;
    end
    
    % SparsePCA scores
    [cards,vars,Z]= sparsePCA(X',m(i));
    omega_SPCA = find( abs(Z) > 0.01 );
    
    [Y_SPCA,~,~] = lle_with_weights(X(omega_SPCA(1:m(i)),:),K_LLE,n_dimensions);
    for j = 1 : n
        error_SPCA(i) = error_SPCA(i) + norm(Y_SPCA(:,j) - ...
            sum(repmat(W(:,j)',n_dimensions,1).*Y_SPCA(:,neighborhood(:,j)),2),2)^2;
    end
    
    [Y_PCoA,~,~] = lle_with_weights(X(omega_PCoA(1:m(i)),:),K_LLE,n_dimensions);
    for j = 1 : n
        error_PCoA(i) = error_PCoA(i) + norm(Y_PCoA(:,j) - ...
            sum(repmat(W(:,j)',n_dimensions,1).*Y_PCoA(:,neighborhood(:,j)),2),2)^2;
    end
    
    [Y_LLE,~,~] = lle_with_weights(X(omega_LLE(1:m(i)),:),K_LLE,n_dimensions);
    for j = 1 : n
        error_LLE(i) = error_LLE(i) + norm(Y_LLE(:,j) - ...
            sum(repmat(W(:,j)',n_dimensions,1).*Y_LLE(:,neighborhood(:,j)),2),2)^2;
    end
    
    [Y_Isomap,~,~] = lle_with_weights(X(omega_Isomap(1:m(i)),:),K_LLE,n_dimensions);
    for j = 1 : n
        error_Isomap(i) = error_Isomap(i) + norm(Y_Isomap(:,j) - ...
            sum(repmat(W(:,j)',n_dimensions,1).*Y_Isomap(:,neighborhood(:,j)),2),2)^2;
    end
    
    
    for t = 1:T
        omega_random = randperm(d,m(i));
        
        %try
            [Y_random,~,~] = lle_with_weights(X(omega_random(1:m(i)),:),K_LLE,n_dimensions);
        %catch
        %    t = t - 1; %#ok<FXSET>
        %    continue
        %end       
        
        for j = 1 : n
            error_random(i) = error_random(i) + norm(Y_random(:,j) - ...
                sum(repmat(W(:,j)',n_dimensions,1).*Y_random(:,neighborhood(:,j)),2),2)^2;
        end
    end
    
end

figure, plot(m,repmat(error_full,size(m)),'Color','b','LineStyle','--','Marker','x','LineWidth',2)
hold on, plot(m,error_PCA,'Color','c','LineStyle','-.','Marker','p','LineWidth',2)
hold on, plot(m,error_SPCA,'Color',[1 0.5 0],'LineStyle','-','Marker','d','LineWidth',2)
hold on, plot(m,error_PCoA,'Color','g','LineStyle','-','Marker','none','LineWidth',2)
hold on, plot(m,error_LLE,'Color','r','LineStyle','-','Marker','+','LineWidth',2)
hold on, plot(m,error_Isomap,'Color','k','LineStyle','-','Marker','v','LineWidth',2)
hold on, plot(m,error_random/T,'Color','m','LineStyle','-','Marker','o','LineWidth',2)
axis tight
axisfortex('','Embedding Dim./Masking Size m','Embedding Error')
legend('Full Data','PCA','SPCA','PCoA','MAPS-LLE','MAPS-Isomap','Random','Location','NorthEast');
title(dataset)
