
%   LLE_greedy_algo: finds the masking pattern according to the MAPS-LLE 
%   algorithm as described in [MAPS].
%
%   Inputs
%   X : dataset matrix
%   K : size of the neighborhood used in LLE
%   m : masking size
%
%   Output
%   omega: masking index set found according to MAPS-LLE algorithm
%
%   BEGIN COPYRIGHT NOTICE
%
%   MAPS code -- (c) 2013-2014 Hamid Dadkhahi
%
%    This code is provided as is, with no guarantees except that 
%    bugs are almost surely present.  Published reports of research 
%    using this code (or a modified version) should cite the 
%    article that describes the algorithm: 
%
%      [MAPS] H. Dadkhahi, M. F. Duarte, Masking Strategies for Image
%      Manifolds, 2014.
%
%    Comments and bug reports are welcome.  Email to hdadkhahi@ecs.umass.edu. 
%    I would also appreciate hearing about how you used this code, 
%    improvements that you have made to it, or translations into other
%    languages.    
%
%    You are free to modify, extend or distribute this code, as long 
%    as this copyright notice is included whole and unchanged. 
%
%    END COPYRIGHT NOTICE

function omega = LLE_graph_algo(X,K,m)

d = size(X,1);
n = size(X,2);

%%

clique = cell(1,n);
s = nchoosek(K+1,2);
A = zeros(s,d,n);

[~,neighborhood] = mapping_weights(X,K);

for i = 1:n
    indices = neighborhood(:,i);
    s = indices(nchoosek(1:K,2));
    t = [repmat(i,K,1) neighborhood(:,i)];
    clique{i} = [s;t];
    A(:,:,i) = ( ( X(:,clique{i}(:,1)) - X(:,clique{i}(:,2)) ).^2 )';
end

%% Generating the LLE mask

omega_comp = 1:d;
omega = [];

value = zeros(1,m);

A_full = sum(A,2);

for i = 1:m
    perccount(i,m)
    
    inner_product = zeros(1,length(omega_comp));
    
    A_masked = sum(A(:,omega,:),2);
    
    for j = 1:length(omega_comp)        
        
        A_temp = A_masked + A(:,omega_comp(j),:);
        
        temp = sum( sqrt(A_temp.*A_full) ,1) ./ ...
            ( sqrt(sum(A_temp,1).*sum(A_full,1)) );
        inner_product(j) = mean(temp);                
    end
    
    [value(i),index] = max(inner_product);
    omega(end+1) = omega_comp(index); %#ok<AGROW>
    omega_comp(index) = [];
end


