
%   isomap_out_of_sample_script.m: reproduces the plots given in Figure 7 
%   of [MAPS].
%
%   BEGIN COPYRIGHT NOTICE
%
%   MAPS code -- (c) 2013-2014 Hamid Dadkhahi
%
%    This code is provided as is, with no guarantees except that 
%    bugs are almost surely present.  Published reports of research 
%    using this code (or a modified version) should cite the 
%    article that describes the algorithm: 
%
%      [MAPS] H. Dadkhahi, M. F. Duarte, Masking Strategies for Image
%      Manifolds, 2014.
%
%    Comments and bug reports are welcome.  Email to hdadkhahi@ecs.umass.edu. 
%    I would also appreciate hearing about how you used this code, 
%    improvements that you have made to it, or translations into other
%    languages.    
%
%    You are free to modify, extend or distribute this code, as long 
%    as this copyright notice is included whole and unchanged. 
%
%    END COPYRIGHT NOTICE

clear all;

warning off; %#ok<WNOFF>

% choose the dataset
%dataset = 'Eyeglasses';
%dataset = 'MNIST';
%dataset = 'Heads';
%dataset = 'Faces'; 
dataset = 'Statue';

% set the parameters according to the chosen dataset
if strcmp(dataset,'Eyeglasses')
    load X_eyedata.mat;
    K_LLE = 12;
    K_Isomap = 12;
    n_dimensions = 2;
elseif strcmp(dataset,'Heads')
    load X_faces.mat;
    K_LLE = 7;
    K_Isomap = 10;
    n_dimensions = 3;
elseif strcmp(dataset,'Faces')
    load X_faces_LLE.mat;
    K_LLE = 10;
    K_Isomap = 9;
    n_dimensions = 3;
elseif strcmp(dataset,'MNIST')
    load X_MNIST.mat;
    K_LLE = 12;
    K_Isomap = 10;
    n_dimensions = 5;
elseif strcmp(dataset,'Statue')
    load X_Statue.mat;
    K_LLE = 11;
    K_Isomap = 8;
    n_dimensions = 3;
else
    display('dataset not found!');
end

X = double(X);
%X = X / max(X(:));
d = size(X,1);
n = size(X,2);

% masking size m
m = 50:50:300;

distances_ambient = L2_distance(X, X, 1);
[~,~,E_isomap,~] = Isomap2(distances_ambient,'k',K_Isomap);
% Generating the secants matrix from neighborhood
[x_ind,y_ind] = find( triu(double(E_isomap) - eye(size(E_isomap,1))) );
A = ( X(:,x_ind) - X(:,y_ind) ).^2;
normA = sum(A,1);
A = A./repmat(normA,d,1);
omega_greedy = greedy_algo(A,m(end));
clear A normA x_ind y_ind E_isomap;

error_full = zeros(1,length(m));
error_Isomap = zeros(1,length(m));
%error_LLE = zeros(1,length(m));
error_random = zeros(1,length(m));

for i = 1:length(m)
    [error_full(i),error_Isomap(i),error_random(i)] = isomap_out_of_sample_function(X,K_Isomap,n_dimensions,omega_greedy(1:m(i)),m(i));    
end

figure, plot(m,error_full,'Color','b','LineStyle','--','Marker','x','LineWidth',2)
%hold on, plot(m,error_PCA,'-.c','LineWidth',2)
%hold on, plot(m,error_PCoA,'-g','LineWidth',2)
%hold on, plot(m,error_LLE,'-r','LineWidth',2)
hold on, plot(m,error_Isomap,'Color','k','LineStyle','-','Marker','v','LineWidth',2)
hold on, plot(m,error_random,'Color','m','LineStyle','-','Marker','o','LineWidth',2)
%axis tight
axis([m(1) m(end) -0.05*error_random(1) error_random(1)])
%axisfortex('','Embedding Dim./Masking Size m','Average OoSE Error')
axisfortex('','Masking Size m','Average OoSE Error')
%legend('Full data','PCA','PCoA Masking','LLE Masking','Isomap Masking','Random Masking')
%legend('Full Data','MAPS-Isomap','Random')
title(dataset)
